@@ -121,6 +121,57 @@ activate_action_reload (GtkAction *action,
 }
 
 
+
+static void
+show_help (GooWindow  *window,
+	   const char *section)
+{
+	GError *err = NULL;  
+
+        gnome_help_display ("goobox", section, &err);
+        
+        if (err != NULL) {
+                GtkWidget *dialog;
+                
+                dialog = _gtk_message_dialog_new (GTK_WINDOW (window),
+						  GTK_DIALOG_DESTROY_WITH_PARENT, 
+						  GTK_STOCK_DIALOG_ERROR,
+						  _("Could not display help"),
+						  err->message,
+						  GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
+						  NULL);
+                gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+                g_signal_connect (G_OBJECT (dialog), "response",
+                                  G_CALLBACK (gtk_widget_destroy),
+                                  NULL);
+                
+                gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+                
+                gtk_widget_show (dialog);
+                
+                g_error_free (err);
+        }
+}
+
+
+void
+activate_action_manual (GtkAction *action, 
+			gpointer   data)
+{
+	GooWindow *window = data;
+	show_help (window, NULL);
+}
+
+
+void
+activate_action_shortcuts (GtkAction *action, 
+			   gpointer   data)
+{
+	GooWindow *window = data;
+	show_help (window, "goobox-shortcuts");
+}
+
+
 void
 activate_action_about (GtkAction *action, 
 		       gpointer   data)
@@ -169,7 +220,7 @@ activate_action_about (GtkAction *action,
 				 "Copyright \xc2\xa9 2004 Free Software Foundation, Inc.",
 				 _("CD player and ripper"),
 				 authors,
-				 documenters,
+				 NULL /*documenters*/,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 logo);
 	if (logo != NULL)
@@ -238,11 +289,69 @@ activate_action_shuffle (GtkAction *action,
 }
 
 
+static void
+sj_watch_func (GPid     pid,
+	       gint     status,
+	       gpointer data)
+{
+	GooWindow *window = data;
+	g_spawn_close_pid (pid);
+	goo_window_set_hibernate (window, FALSE);
+}
+
+
+static gboolean
+exec_sj (const char  *command_line,
+	 GError     **error,
+	 gpointer     data)
+{
+	gboolean   retval;
+	gchar    **argv = NULL;
+	GPid       child_pid;
+
+	g_return_val_if_fail (command_line != NULL, FALSE);
+	
+	if (!g_shell_parse_argv (command_line,
+				 NULL, &argv,
+				 error))
+		return FALSE;
+
+	retval = g_spawn_async (NULL,
+				argv,
+				NULL,
+				G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD,
+				NULL,
+				NULL,
+				&child_pid,
+				error);
+	g_strfreev (argv);
+
+	g_child_watch_add (child_pid,
+			   sj_watch_func,
+			   data);
+
+	return retval;
+}
+
+
 void
 activate_action_extract (GtkAction *action, 
 			 gpointer   data)
 {
-	dlg_extract (GOO_WINDOW (data));
+	GooWindow *window = data;
+
+	if (preferences_get_use_sound_juicer ()) {
+		GError *error = NULL;
+
+		goo_window_set_hibernate (window, TRUE);		
+
+		if (! exec_sj ("sound-juicer", &error, data))
+			_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
+							   _("Could not execute command"), 
+							   &error);
+
+	} else
+		dlg_extract (window);
 }
 
 
