@@ -58,7 +58,7 @@ path_is_file (const gchar *path)
 	if (! path || ! *path) return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -82,10 +82,11 @@ path_is_dir (const gchar *path)
 	gboolean is_dir;
 	gchar *escaped;
 
-	if (! path || ! *path) return FALSE; 
+	if (! path || ! *path) 
+		return FALSE; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -155,7 +156,7 @@ get_file_size (const gchar *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info,
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -182,7 +183,7 @@ get_file_mtime (const gchar *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -209,7 +210,7 @@ get_file_ctime (const gchar *path)
 	if (! path || ! *path) return 0; 
 
 	info = gnome_vfs_file_info_new ();
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	result = gnome_vfs_get_file_info (escaped, 
 					  info, 
 					  (GNOME_VFS_FILE_INFO_DEFAULT 
@@ -244,7 +245,7 @@ file_copy (const gchar *from,
 		return FALSE;
 
 	dest_dir = remove_level_from_path (to);
-	if (! ensure_dir_exists (dest_dir, 0755)) {
+	if (ensure_dir_exists (dest_dir, 0755) != GNOME_VFS_OK) {
 		g_free (dest_dir);
 		fclose (fin);
 		return FALSE;
@@ -377,40 +378,60 @@ remove_ending_separator (const gchar *path)
 }
 
 
-gboolean 
+GnomeVFSResult
 ensure_dir_exists (const gchar *a_path,
 		   mode_t       mode)
 {
-	if (! a_path) return FALSE;
+	char *path;
+	char *p;
+	char *scheme;
 
-	if (! path_is_dir (a_path)) {
-		gchar *path = g_strdup (a_path);
-		gchar *p = path;
+	if (! a_path) 
+		return GNOME_VFS_ERROR_BAD_PARAMETERS;
+	
+	if (path_is_dir (a_path))
+		return GNOME_VFS_OK;
 
-		while (*p != '\0') {
-			p++;
-			if ((*p == '/') || (*p == '\0')) {
-				gboolean end = TRUE;
+	path = g_strdup (a_path);
+	p = path;
 
-				if (*p != '\0') {
-					*p = '\0';
-					end = FALSE;
-				}
+	scheme = gnome_vfs_get_uri_scheme (a_path);
+	if (scheme != NULL) {
+		p += strlen (scheme);
+		if (strncmp (p, "://", 3) == 0)
+			p += 3;
+	}
+	
+	while (*p != '\0') {
+		gboolean end;
 
-			if (! path_is_dir (path)) {
-					if (mkdir (path, mode) < 0) {
-						g_warning ("directory creation failed: %s.", path);
-						g_free (path);
-						return FALSE;
-					}
-				}
-				if (! end) *p = '/';
+		p++;
+		if ((*p != '/') && (*p != '\0')) 
+			continue;
+
+		end = TRUE;
+		if (*p != '\0') {
+			*p = '\0';
+			end = FALSE;
+		}
+			
+		if (! path_is_dir (path)) {
+			GnomeVFSResult result;
+			result = gnome_vfs_make_directory (path, mode);
+			if (result != GNOME_VFS_OK) {
+				g_free (path);
+				return result;
 			}
 		}
-		g_free (path);
-	}
 
-	return TRUE;
+		if (! end) 
+			*p = '/';
+	}
+	
+	g_free (scheme);
+	g_free (path);
+	
+	return GNOME_VFS_OK;
 }
 
 
@@ -545,7 +566,7 @@ escape_str (const char *str,
 char*
 shell_escape (const char *filename)
 {
-	return escape_str (filename, "$\'`\"\\!?* ()[]&|@#:;");
+	return escape_str (filename, "$\'`\"\\!?* ()[]<>&|@#:;");
 }
 
 
@@ -837,7 +858,7 @@ get_dest_free_space (const char  *path)
 	GnomeVFSResult    result;
 	GnomeVFSFileSize  ret_val;
 
-	escaped = gnome_vfs_escape_path_string (path);
+	escaped = gnome_vfs_escape_host_and_path_string (path);
 	uri = gnome_vfs_uri_new (escaped);
 	g_free (escaped);
 
@@ -1140,3 +1161,107 @@ file_list__get_prev_field (const char *line,
 
 	return g_strndup (f_start + 1, f_end - f_start);
 }
+
+
+GnomeVFSURI *
+new_uri_from_path (const char *path)
+{
+	char        *escaped;
+	char        *uri_txt;
+	GnomeVFSURI *uri;
+
+	escaped = escape_uri (path);
+	if (escaped[0] == '/')
+		uri_txt = g_strconcat ("file://", escaped, NULL);
+	else
+		uri_txt = g_strdup (escaped);
+
+	uri = gnome_vfs_uri_new (uri_txt);
+
+	g_free (uri_txt);
+	g_free (escaped);
+
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	return uri;
+}
+
+
+
+static gboolean
+valid_filename_char (char c) 
+{
+	/* "$\'`\"\\!?* ()[]<>&|@#:;" */
+	return strchr ("/\\!?*:;'`\"", c) == NULL;
+}
+
+
+/* Remove special characters from a track title in order to make it a 
+ * valid filename. */
+char*
+tracktitle_to_filename (const char *trackname)
+{
+	char       *filename, *f, *f2;
+	const char *t;
+	gboolean    add_space;
+
+	if (trackname == NULL)
+		return NULL;
+
+	filename = g_new (char, strlen (trackname) + 1);
+
+	/* Substitute invalid characters with spaces. */
+	f = filename;
+	t = trackname;
+	while (*t != 0) {
+		gboolean invalid_char = FALSE;
+
+		while ((*t != 0) && ! valid_filename_char (*t)) {
+			invalid_char = TRUE;
+			t++;
+		}
+
+		if (invalid_char)
+			*f++ = ' ';
+		
+		*f = *t;
+
+		if (*t != 0) {
+			f++;
+			t++;
+		}
+	}
+	*f = 0;
+
+	/* Remove double spaces. */
+	add_space = FALSE;
+	for (f = f2 = filename; *f != 0; f++)
+		if (*f != ' ') {
+			if (add_space) {
+				*f2++ = ' ';
+				add_space = FALSE;
+			}
+			*f2 = *f;
+			f2++;
+		} else
+			add_space = TRUE;
+	*f2 = 0;
+
+	return filename;
+}
+
+
+gboolean
+is_program_in_path (const char *program_name)
+{
+	char *str;
+        int   result = FALSE;
+	
+        str = g_find_program_in_path (program_name);
+        if (str != NULL) {
+                g_free (str);
+                result = TRUE;
+        }
+	
+        return result;
+}
