@@ -272,10 +272,12 @@ create_pipeline (GooPlayerCD *player)
 	priv->volume = gst_element_factory_make ("volume", "volume");
 
 	queue = gst_element_factory_make ("queue", "queue");
+	/* FIXME
 	g_object_set (G_OBJECT (queue), 
 		      "max-size-bytes", QUEUE_SIZE,
 		      NULL);
-	
+	*/
+
 	sink = gst_gconf_get_default_audio_sink ();
 	audio_thread = gst_thread_new ("a_decoder_thread");
 
@@ -415,10 +417,12 @@ list_thread (void *thread_data)
 
 	gst_pad_query (priv->source_pad, GST_QUERY_TOTAL, &(priv->track_format), &(priv->n_tracks));
 
+	debug (DEBUG_INFO, "TOTAL TRACKS: %" G_GINT64_FORMAT "\n", priv->n_tracks);
+
 	gst_pad_query (priv->source_pad, GST_QUERY_TOTAL, &(priv->sector_format), &total_sectors);
 	total_sectors += TOC_OFFSET;
 	
-	debug (DEBUG_INFO, "TOTAL SECTORS: %" G_GINT64_FORMAT "\n", total_sectors + from_sector);
+	debug (DEBUG_INFO, "TOTAL SECTORS: %" G_GINT64_FORMAT "\n", total_sectors);
 	
 	gst_pad_convert (priv->source_pad, 
 			 priv->sector_format, 
