@@ -23,23 +23,27 @@
 #include <config.h>
 #include <string.h>
 #include <gnome.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include "goo-player-info.h"
 #include "goo-marshal.h"
 #include "glib-utils.h"
 #include "goo-stock.h"
+#include "file-utils.h"
 
 #define SPACING 0
 #define TITLE1_FORMAT "<span size='large' weight='bold'>%s</span>"
 #define TITLE2_FORMAT "%s"
 #define TITLE3_FORMAT "<i>%s</i>"
 #define TIME_FORMAT "%s"
-#define PLAYING_FORMAT "%s"
+#define PLAYING_FORMAT "[ <small>%s</small> ]"
 #define SCALE_WIDTH 150
 #define COVER_SIZE 80
-#define MIN_WIDTH 400
+#define MIN_WIDTH 420
+#define MIN_CHARS 45
 #define UPDATE_TIMEOUT 50
 
 struct _GooPlayerInfoPrivateData {
+	GooWindow   *window;
 	GooPlayer   *player;
 	GtkWidget   *title1_label;
 	GtkWidget   *title2_label;
@@ -49,6 +53,7 @@ struct _GooPlayerInfoPrivateData {
 	GtkWidget   *time_scale;
 	GtkWidget   *cover_image;
 	GtkWidget   *cover_button;
+	GtkWidget   *cover_sep;
 	GtkTooltips *tips;
 	char         current_time[64];
 	char         total_time[64];
@@ -67,6 +72,12 @@ enum {
 static GtkHBoxClass *parent_class = NULL;
 static guint goo_player_info_signals[LAST_SIGNAL] = { 0 };
 
+enum { TARGET_URL };
+static GtkTargetEntry target_table[] = {
+	{ "text/uri-list", 0, TARGET_URL }
+};
+static guint n_targets = sizeof (target_table) / sizeof (target_table[0]);
+
 static void goo_player_info_class_init  (GooPlayerInfoClass *class);
 static void goo_player_info_init        (GooPlayerInfo *player);
 static void goo_player_info_finalize    (GObject *object);
@@ -154,6 +165,11 @@ set_label (GtkWidget     *label,
 	char *e_text;
 	char *markup;
 
+	if ((text == NULL) || (*text == '\0')) {
+		gtk_label_set_text (GTK_LABEL (label), "");
+		return;
+	}
+
 	e_text = g_markup_escape_text (text, -1);
 	markup = g_strdup_printf (format, e_text);
 	g_free (e_text);
@@ -217,7 +233,7 @@ time_scale_value_changed_cb (GtkRange      *range,
 		current_time = priv->song_length * (new_value / 100.0);
 
 		set_time_string (priv->current_time, current_time);
-		sprintf (priv->time, "%s", priv->current_time);
+		sprintf (priv->time, _("%s / %s"), priv->total_time, priv->current_time);
 		set_time (info, priv->time);
 
 		return;
@@ -242,7 +258,7 @@ update_time_label_cb (gpointer data)
 	
 	current_time = priv->song_length * new_value;
 	set_time_string (priv->current_time, current_time);
-	sprintf (priv->time, "%s", priv->current_time);
+	sprintf (priv->time, _("%s / %s"), priv->current_time, priv->total_time);
 	set_time (info, priv->time);
 
 	priv->update_id = g_timeout_add (UPDATE_TIMEOUT,
@@ -289,6 +305,121 @@ cover_button_clicked_cb (GtkWidget     *button,
 }
 
 
+/* -- drag and drop -- */
+
+
+static char *
+get_path_from_url (char *url)
+{
+	GnomeVFSURI *uri;
+	char        *escaped;
+	char        *path;
+
+	if (url == NULL)
+		return NULL;
+
+	uri = gnome_vfs_uri_new (url);
+	escaped = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
+	path = gnome_vfs_unescape_string (escaped, NULL);
+
+	gnome_vfs_uri_unref (uri);
+	g_free (escaped);
+
+	return path;
+}
+
+
+static GList *
+get_file_list_from_dnd_data (gchar *url_list)
+{
+	GList    *list = NULL;
+	int       i;
+	char     *url_start, *url_end;
+	gboolean  dnd_valid = FALSE;
+
+	i = 0;
+	url_start = url_list;
+	while (url_list[i] != '\0')	{
+		char *url;
+
+		while ((url_list[i] != '\0')
+		       && (url_list[i] != '\r')
+		       && (url_list[i] != '\n')) i++;
+
+		url_end = url_list + i;
+		if (strncmp (url_start, "file:", 5) == 0) {
+			url_start += 5;
+			if ((url_start[0] == '/') 
+			    && (url_start[1] == '/')) url_start += 2;
+
+			dnd_valid = TRUE;
+
+			url = g_strndup (url_start, url_end - url_start);
+			list = g_list_prepend (list, get_path_from_url (url));
+			g_free (url);
+
+			while ((url_list[i] != '\0')
+			       && ((url_list[i] == '\r') 
+				   || (url_list[i] == '\n'))) i++;
+			url_start = url_list + i;
+
+		} else {
+			while ((url_list[i] != '\0')
+			       && ((url_list[i] == '\r') 
+				   || (url_list[i] == '\n'))) i++;
+			url_start = url_list + i;
+			if (url_list[i] == '\0' && !dnd_valid) {
+				path_list_free (list);
+				return NULL;
+			}
+		}
+	}
+	
+	if (!dnd_valid) {
+		path_list_free (list);
+		return NULL;
+	}
+
+	return g_list_reverse (list);
+}
+
+
+void  
+cover_button_drag_data_received  (GtkWidget          *widget,
+				  GdkDragContext     *context,
+				  gint                x,
+				  gint                y,
+				  GtkSelectionData   *data,
+				  guint               dnd_info,
+				  guint               time,
+				  gpointer            extra_data)
+{
+	GooPlayerInfo *info = extra_data;
+	GList         *list;
+	char          *cover_filename;
+
+	if (! ((data->length >= 0) && (data->format == 8))) {
+		gtk_drag_finish (context, FALSE, FALSE, time);
+		return;
+	}
+
+	gtk_drag_finish (context, TRUE, FALSE, time);
+
+	list = get_file_list_from_dnd_data ((char *)data->data);
+	if (list == NULL)
+		return;
+
+	cover_filename = list->data;
+	if (cover_filename == NULL)
+		return;
+
+	goo_window_set_cover_image (info->priv->window, cover_filename);
+
+	if (list != NULL) 
+		path_list_free (list);
+}
+
+
 static void 
 goo_player_info_init (GooPlayerInfo *info)
 {
@@ -323,13 +454,24 @@ goo_player_info_init (GooPlayerInfo *info)
 	gtk_misc_set_alignment (GTK_MISC (priv->title3_label), 0.0, 0.5);
 	gtk_label_set_selectable (GTK_LABEL (priv->title3_label), TRUE);
 
-	/* time */
+#ifdef HAVE_GTK_2_5
+	gtk_label_set_ellipsize (GTK_LABEL (priv->title1_label),
+				 PANGO_ELLIPSIZE_END);
+	gtk_label_set_width_chars (GTK_LABEL (priv->title2_label), MIN_CHARS);
+
+	gtk_label_set_ellipsize (GTK_LABEL (priv->title2_label),
+				 PANGO_ELLIPSIZE_END);
+	gtk_label_set_width_chars (GTK_LABEL (priv->title2_label), MIN_CHARS);
+	
+	gtk_label_set_ellipsize (GTK_LABEL (priv->title3_label),
+				 PANGO_ELLIPSIZE_END);
+	gtk_label_set_width_chars (GTK_LABEL (priv->title3_label), MIN_CHARS);
+#endif
+
+	/* Time */
 
 	time_box = gtk_hbox_new (FALSE, 6);
 
-	priv->time_label = gtk_label_new (NULL);
-	gtk_widget_set_no_show_all (priv->time_label, TRUE);
-
 	priv->time_scale = gtk_hscale_new_with_range (0.0, 1.0, 0.01);
 	gtk_range_set_increments (GTK_RANGE (priv->time_scale), 0.01, 0.1);
 	gtk_scale_set_draw_value (GTK_SCALE (priv->time_scale), FALSE);
@@ -337,11 +479,14 @@ goo_player_info_init (GooPlayerInfo *info)
 	gtk_range_set_update_policy (GTK_RANGE (priv->time_scale), GTK_UPDATE_DISCONTINUOUS);
 	gtk_widget_set_no_show_all (priv->time_scale, TRUE);
 
+	priv->time_label = gtk_label_new (NULL);
+	gtk_widget_set_no_show_all (priv->time_label, TRUE);
+
 	priv->playing_label = gtk_label_new (NULL);
 	gtk_widget_set_no_show_all (priv->playing_label, TRUE);
 
-	gtk_box_pack_start (GTK_BOX (time_box), priv->time_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (time_box), priv->time_scale, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (time_box), priv->time_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (time_box), priv->playing_label, FALSE, FALSE, 0);
 
 	/* Image */
@@ -358,6 +503,16 @@ goo_player_info_init (GooPlayerInfo *info)
 			  G_CALLBACK (cover_button_clicked_cb),
 			  info);
 
+	gtk_drag_dest_set (priv->cover_button,
+			   GTK_DEST_DEFAULT_ALL,
+			   target_table, n_targets,
+			   GDK_ACTION_COPY);
+	
+	g_signal_connect (G_OBJECT (priv->cover_button), 
+			  "drag_data_received",
+			  G_CALLBACK (cover_button_drag_data_received), 
+			  info);
+
 	priv->cover_image = gtk_image_new_from_stock (GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
 	gtk_widget_set_size_request (priv->cover_image, COVER_SIZE, COVER_SIZE);
 	gtk_widget_show (priv->cover_image);
@@ -365,11 +520,10 @@ goo_player_info_init (GooPlayerInfo *info)
 
 	gtk_box_pack_start (GTK_BOX (info), priv->cover_button, FALSE, FALSE, 0);
 
-	/*FIXME
-	gtk_box_pack_start (GTK_BOX (info), 
-			    gtk_vseparator_new (),
-			    FALSE, FALSE, 0);
-	*/
+	priv->cover_sep = gtk_vseparator_new ();
+	gtk_box_pack_start (GTK_BOX (info), priv->cover_sep, FALSE, FALSE, 6);
+	gtk_widget_show (priv->cover_sep);
+	gtk_widget_set_no_show_all (priv->cover_sep, TRUE);
 
 	/**/
 
@@ -430,11 +584,16 @@ goo_player_info_finalize (GObject *object)
 
 
 GtkWidget *
-goo_player_info_new (GooPlayer *player)
+goo_player_info_new (GooWindow *window,
+		     GooPlayer *player)
 {
 	GooPlayerInfo *info;
 	
 	info = GOO_PLAYER_INFO (g_object_new (GOO_TYPE_PLAYER_INFO, NULL));
+
+	if (window != NULL) 
+		info->priv->window = window;
+
 	if (player != NULL) {
 		g_object_ref (player);
 		info->priv->player = player;
@@ -550,7 +709,7 @@ goo_player_info_update_state (GooPlayerInfo  *info)
 			set_title1 (info, _("Reading CD"));
 			set_title2 (info, "");
 
-		}  else {
+		} else {
 			const char *title, *subtitle;
 
 			title = goo_player_get_title (priv->player);
@@ -596,7 +755,7 @@ goo_player_info_set_time (GooPlayerInfo  *info,
 		return;
 
 	set_time_string (priv->current_time, current_time);
-	sprintf (priv->time, "%s", priv->current_time);
+	sprintf (priv->time, _("%s / %s"), priv->current_time, priv->total_time);
 	set_time (info, priv->time);
 
 	g_signal_handlers_block_by_data (priv->time_scale, info);
@@ -617,8 +776,11 @@ void
 goo_player_info_set_cover (GooPlayerInfo  *info,
 			   GdkPixbuf      *cover)
 {
-	if (cover != NULL)
+	if (cover != NULL) {
 		gtk_image_set_from_pixbuf (GTK_IMAGE (info->priv->cover_image), cover);
-	else
+		gtk_widget_hide (info->priv->cover_sep);
+	} else {
 		gtk_image_set_from_stock (GTK_IMAGE (info->priv->cover_image), GOO_STOCK_NO_COVER, GTK_ICON_SIZE_DIALOG);
+		gtk_widget_show (info->priv->cover_sep);
+	}
 }
