@@ -3,7 +3,7 @@
 /*
  *  Goo
  *
- *  Copyright (C) 2004 Free Software Foundation, Inc.
+ *  Copyright (C) 2004, 2005 Free Software Foundation, Inc.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -69,6 +69,8 @@
 #define PLAYER_CHECK_RATE 100
 #define COVER_SIZE 80
 #define IDLE_TIMEOUT 200
+#define FALLBACK_ICON_SIZE 16
+
 
 struct _GooWindowPrivateData {
 	GtkUIManager    *ui;
@@ -77,6 +79,9 @@ struct _GooWindowPrivateData {
 	GtkWidget       *toolbar;
 	GtkWidget       *list_expander;
 
+	WindowSortMethod sort_method;
+	GtkSortType      sort_type;
+
 	GtkWidget       *statusbar;
 	GtkWidget       *file_popup_menu;
 	GtkWidget       *tray_popup_menu;
@@ -119,12 +124,10 @@ struct _GooWindowPrivateData {
 
 	gboolean         exiting;
 	guint            check_id;
-
 	GList           *url_list;
-
 	GtkWidget       *preview;
-
 	int              pos_x, pos_y;
+	gboolean         hibernate;
 };
 
 static int icon_size = 0;
@@ -273,7 +276,7 @@ window_update_sensitivity (GooWindow *window)
 	sel_not_null      = n_selected > 0;
 	one_file_selected = n_selected == 1;
 	state             = goo_player_get_state (priv->player);
-	error             = state == GOO_PLAYER_STATE_ERROR;
+	error             = (state == GOO_PLAYER_STATE_ERROR) || priv->hibernate;
 	playing           = state == GOO_PLAYER_STATE_PLAYING;
 	paused            = state == GOO_PLAYER_STATE_PAUSED;
 	stopped           = state == GOO_PLAYER_STATE_STOPPED;
@@ -298,7 +301,91 @@ window_update_sensitivity (GooWindow *window)
 	set_sensitive (window, "Repeat", play_all);
 	set_sensitive (window, "Shuffle", play_all);
 
-	gtk_widget_set_sensitive (priv->list_view, discid != NULL);
+	gtk_widget_set_sensitive (priv->list_view, !priv->hibernate && (discid != NULL));
+
+	set_sensitive (window, "Eject", !priv->hibernate);
+	set_sensitive (window, "Preferences", !priv->hibernate);
+}
+
+
+static GdkPixbuf *
+create_void_icon (GooWindow *window)
+{
+	GtkSettings  *settings;
+	int           width, height, icon_size;
+	GdkPixbuf    *icon;
+
+	settings = gtk_settings_get_for_screen (gtk_widget_get_screen (GTK_WIDGET (window)));
+
+	if (gtk_icon_size_lookup_for_settings (settings, GTK_ICON_SIZE_MENU, &width, &height))
+		icon_size = MAX (width, height);
+	else
+		icon_size = FALLBACK_ICON_SIZE;
+
+	icon = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, icon_size, icon_size);
+	gdk_pixbuf_fill (icon, 0x00000000);
+
+	return icon;
+}
+
+
+static gboolean
+get_iter_from_track_number (GooWindow   *window,
+			    int          track_number,
+			    GtkTreeIter *iter)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeModel         *model = GTK_TREE_MODEL (priv->list_store);
+
+	if (! gtk_tree_model_get_iter_first (model, iter))
+		return FALSE;
+	
+	do {
+		SongInfo *song;
+		gtk_tree_model_get (model, iter, COLUMN_SONG_INFO, &song, -1);
+		if (song->number == track_number) {
+			song_info_unref (song);
+			return TRUE;
+		}
+		song_info_unref (song);
+	} while (gtk_tree_model_iter_next (model, iter));
+
+	return FALSE;
+}
+
+
+static void
+set_song_icon (GooWindow  *window,
+	       int         track_number,
+	       const char *stock_id)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeIter           iter;
+	GdkPixbuf            *icon;
+
+	if (!get_iter_from_track_number (window, track_number, &iter)) 
+		return;
+
+	if (stock_id != NULL)
+		icon = gtk_widget_render_icon (GTK_WIDGET (window), 
+					       stock_id, 
+					       GTK_ICON_SIZE_MENU, 
+					       NULL);
+	else
+		icon = create_void_icon (window);
+	gtk_list_store_set (priv->list_store, &iter,
+			    COLUMN_ICON, icon,
+			    -1);
+	g_object_unref (icon);
+}
+
+
+static void
+set_current_song_icon (GooWindow  *window,
+		       const char *stock_id)
+{
+	if (window->priv->current_song != NULL)
+		set_song_icon (window, window->priv->current_song->number, stock_id);
 }
 
 
@@ -311,6 +398,7 @@ update_list_idle (gpointer callback_data)
 	GList                *file_list;
 	GList                *scan;
 	int                   i, n = FILES_TO_PROCESS_AT_ONCE;
+	GdkPixbuf            *icon;
 
 	if (priv->update_timeout_handle != 0) {
 		g_source_remove (priv->update_timeout_handle);
@@ -330,6 +418,8 @@ update_list_idle (gpointer callback_data)
 	data->file_list = scan->next;
 	scan->next = NULL;
 
+	icon = create_void_icon (window);
+
 	for (scan = file_list; scan; scan = scan->next) {
 		SongInfo    *song = scan->data;
 		GtkTreeIter  iter;
@@ -341,6 +431,7 @@ update_list_idle (gpointer callback_data)
 		gtk_list_store_set (priv->list_store, &iter,
 				    COLUMN_SONG_INFO, song,
 				    COLUMN_NUMBER, song->number + 1, /*FIXME*/
+				    COLUMN_ICON, icon,
 				    COLUMN_TIME, time_s,
 				    COLUMN_TITLE, song->title,
 				    COLUMN_ARTIST, song->artist,
@@ -348,6 +439,8 @@ update_list_idle (gpointer callback_data)
 		g_free (time_s);
 	}
 
+	g_object_unref (icon);
+
 	if (gtk_events_pending ())
 		gtk_main_iteration_do (TRUE);
 
@@ -362,7 +455,7 @@ update_list_idle (gpointer callback_data)
 void
 goo_window_update (GooWindow *window)
 {
-	goo_player_stop (window->priv->player);
+	goo_window_stop (window);
 	goo_player_update (window->priv->player);
 }
 
@@ -450,10 +543,13 @@ goo_window_update_titles (GooWindow *window)
 		
 		gtk_tree_model_get (model, &iter, COLUMN_SONG_INFO, &song, -1);
 		new_song = find_song_from_number (song_list, song->number);
+		song_info_unref (song);
+
 		if (new_song == NULL)
 			continue;
 
 		gtk_list_store_set (priv->list_store, &iter,
+				    COLUMN_SONG_INFO, new_song,
 				    COLUMN_TITLE, new_song->title,
 				    COLUMN_ARTIST, new_song->artist,
 				    -1);
@@ -462,7 +558,8 @@ goo_window_update_titles (GooWindow *window)
 		if ((priv->current_song != NULL) 
 		    && (new_song->number == priv->current_song->number)) {
 			song_info_unref (priv->current_song);
-			priv->current_song = song_info_copy (new_song);
+			song_info_ref (new_song);
+			priv->current_song = new_song;
 		}
 
 	} while (gtk_tree_model_iter_next (model, &iter));
@@ -585,6 +682,16 @@ add_columns (GtkTreeView *treeview)
 
 		if (i == COLUMN_ARTIST)
 			gtk_tree_view_column_set_visible (column, FALSE);
+
+		if (i == COLUMN_TITLE) {
+			GValue  value = { 0, };
+			g_value_init (&value, PANGO_TYPE_ELLIPSIZE_MODE);
+			g_value_set_enum (&value, PANGO_ELLIPSIZE_END);
+			g_object_set_property (G_OBJECT (renderer), 
+					       "ellipsize", 
+					       &value);
+			g_value_unset (&value);
+		}
 	}
 }
 
@@ -694,6 +801,60 @@ print_playlist (GooWindow *window)
 }
 
 
+static int
+get_track_number_from_position (GooWindow   *window,
+				int          pos)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeModel         *model = GTK_TREE_MODEL (priv->list_store);
+	GtkTreeIter           iter;
+	int                   i = 0;
+
+	if (! gtk_tree_model_get_iter_first (model, &iter))
+		return -1;
+	
+	do {
+		SongInfo *song;
+		int       n;
+		gtk_tree_model_get (model, &iter, COLUMN_SONG_INFO, &song, -1);
+		n = song->number;
+		song_info_unref (song);
+		if (i == pos)
+			return n;
+		i++;
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return -1;
+}
+
+
+static int
+get_position_from_track_number (GooWindow   *window,
+				int          track_number)
+{
+	GooWindowPrivateData *priv = window->priv;
+	GtkTreeModel         *model = GTK_TREE_MODEL (priv->list_store);
+	GtkTreeIter           iter;
+	int                   pos = 0;
+
+	if (! gtk_tree_model_get_iter_first (model, &iter))
+		return -1;
+	
+	do {
+		SongInfo *song;
+		int       n;
+		gtk_tree_model_get (model, &iter, COLUMN_SONG_INFO, &song, -1);
+		n = song->number;
+		song_info_unref (song);
+		if (n == track_number)
+			return pos;
+		pos++;
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	return -1;
+}
+
+
 static void
 create_playlist (GooWindow *window,
 		 gboolean   play_all,
@@ -716,14 +877,15 @@ create_playlist (GooWindow *window,
 	playlist = NULL;
 
 	if (priv->current_song != NULL)
-		pos = priv->current_song->number;
+		pos = get_position_from_track_number (window, priv->current_song->number);
 	n = g_list_length (priv->song_list);
 
 	for (i = 0; i < n; i++, pos = (pos + 1) % n) {
+		int track_number = get_track_number_from_position (window, pos);
 		if ((priv->current_song != NULL) 
-		    && (priv->current_song->number == pos))
+		    && (priv->current_song->number == track_number))
 			continue;
-		playlist = g_list_prepend (playlist, GINT_TO_POINTER (pos));
+		playlist = g_list_prepend (playlist, GINT_TO_POINTER (track_number));
 	}
 
 	playlist = g_list_reverse (playlist);
@@ -735,21 +897,29 @@ create_playlist (GooWindow *window,
 
 		while (playlist != NULL) {
 			GList *item;
-
 			pos = g_rand_int_range (grand, 0, len--);
 			item = g_list_nth (playlist, pos);
 			playlist = g_list_remove_link (playlist, item);
 			random_list = g_list_concat (random_list, item);
 		}
-		g_rand_free (grand);
-
 		playlist = random_list;
+
+		g_rand_free (grand);
 	} 
 
 	priv->playlist = playlist;
 }
 
 
+static void
+play_song (GooWindow *window,
+	   gint       track_number)
+{
+	GooWindowPrivateData *priv = window->priv;
+	goo_player_seek_song (priv->player, track_number);
+}
+
+
 static void
 play_next_song_in_playlist (GooWindow *window)
 {
@@ -777,10 +947,10 @@ play_next_song_in_playlist (GooWindow *window)
 	print_playlist (window);
 
 	if (next == NULL)
-		goo_player_stop (priv->player);
+		goo_window_stop (window);
 	else {
 		int pos = GPOINTER_TO_INT (next->data);
-		goo_player_seek_song (priv->player, pos);
+		play_song (window, pos);
 		priv->playlist = g_list_remove_link (priv->playlist, next);
 		g_list_free (next);
 	}
@@ -884,10 +1054,8 @@ goo_window_unrealize (GtkWidget *widget)
 	if (playlist_visible)
 		save_window_size (window);
 
-	/* FIXME
-	preferences_set_sort_method (window->sort_method);
-	preferences_set_sort_type (window->sort_type);
-	*/
+	preferences_set_sort_method (priv->sort_method);
+	preferences_set_sort_type (priv->sort_type);
 
 	GTK_WIDGET_CLASS (parent_class)->unrealize (widget);
 }
@@ -980,7 +1148,7 @@ sort_by_name (const char *s1,
 		return 1;
 	else if (s2 == NULL)
 		return -1;
-	else
+	else 
 		return g_utf8_collate (s1, s2);
 }
 
@@ -992,16 +1160,24 @@ title_column_sort_func (GtkTreeModel *model,
 			gpointer      user_data)
 {
 	SongInfo *song1 = NULL, *song2 = NULL;
+	int       retval = -1;
+
         gtk_tree_model_get (model, a, COLUMN_SONG_INFO, &song1, -1);
         gtk_tree_model_get (model, b, COLUMN_SONG_INFO, &song2, -1);
+
 	if ((song1 == NULL) && (song2 == NULL))
-		return 0;
+		retval = 0;
 	else if (song1 == NULL)
-		return 1;
+		retval = 1;
 	else if (song2 == NULL)
-		return -1;
+		retval = -1;
 	else
-		return sort_by_name (song1->title, song2->title);
+		retval = sort_by_name (song1->title, song2->title);
+
+	song_info_unref (song1);
+	song_info_unref (song2);
+
+	return retval;
 }
 
 
@@ -1012,20 +1188,29 @@ artist_column_sort_func (GtkTreeModel *model,
 			 gpointer      user_data)
 {
 	SongInfo *song1 = NULL, *song2 = NULL;
+	int       retval = -1;
+
         gtk_tree_model_get (model, a, COLUMN_SONG_INFO, &song1, -1);
         gtk_tree_model_get (model, b, COLUMN_SONG_INFO, &song2, -1);
+
 	if ((song1 == NULL) && (song2 == NULL))
-		return 0;
+		retval = 0;
 	else if (song1 == NULL)
-		return 1;
+		retval = 1;
 	else if (song2 == NULL)
-		return -1;
+		retval = -1;
 	else {
 		int result = sort_by_name (song1->artist, song2->artist);
 		if (result == 0)
-			return sort_by_number (song1->number, song2->number);
-		return result;
+			retval = sort_by_number (song1->number, song2->number);
+		else
+			retval = result;
 	}
+
+	song_info_unref (song1);
+	song_info_unref (song2);
+
+	return retval;
 }
 
 
@@ -1036,16 +1221,24 @@ time_column_sort_func (GtkTreeModel *model,
 		       gpointer      user_data)
 {
 	SongInfo *song1 = NULL, *song2 = NULL;
+	int       retval = -1;
+
         gtk_tree_model_get (model, a, COLUMN_SONG_INFO, &song1, -1);
         gtk_tree_model_get (model, b, COLUMN_SONG_INFO, &song2, -1);
+
 	if ((song1 == NULL) && (song2 == NULL))
-		return 0;
+		retval = 0;
 	else if (song1 == NULL)
-		return 1;
+		retval = 1;
 	else if (song2 == NULL)
-		return -1;
+		retval = -1;
 	else 
-		return sort_by_number (song1->time, song2->time);
+		retval = sort_by_number (song1->time, song2->time);
+
+	song_info_unref (song1);
+	song_info_unref (song2);
+
+	return retval;
 }
 
 
@@ -1166,7 +1359,7 @@ goo_window_select_current_song (GooWindow *window)
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->list_view));
 	gtk_tree_selection_unselect_all (selection);
 
-	pos = priv->current_song->number;
+	pos = get_position_from_track_number (window, priv->current_song->number);
 	if (pos == -1)
 		return;
 
@@ -1200,7 +1393,11 @@ goo_window_set_current_song (GooWindow *window,
 	GooWindowPrivateData *priv = window->priv;
 	SongInfo *song;
 
+	if (window->priv->hibernate)
+		return;
+
 	if (priv->current_song != NULL) {
+		set_current_song_icon (window, NULL);
 		song_info_unref (priv->current_song);	
 		priv->current_song = NULL;
 	}
@@ -1343,6 +1540,7 @@ player_done_cb (GooPlayer       *player,
 		goo_window_update_list (window);
 		goo_window_update_cover (window);
 		window_update_title (window);
+		set_current_song_icon (window, NULL);
 		if (AutoPlay) {
 			AutoPlay = FALSE;
 			g_idle_add (autoplay_cb, window);
@@ -1357,6 +1555,7 @@ player_done_cb (GooPlayer       *player,
 	case GOO_PLAYER_ACTION_SEEK_SONG:
 		goo_window_set_current_song (window, goo_player_get_current_song (priv->player));
 		goo_window_select_current_song (window);
+		set_current_song_icon (window, GOO_STOCK_PLAY);
 		break;
 	case GOO_PLAYER_ACTION_PLAY:
 	case GOO_PLAYER_ACTION_STOP:
@@ -1368,11 +1567,16 @@ player_done_cb (GooPlayer       *player,
 					   GOO_STOCK_PLAY,
 					   "/MenuBar/CDMenu/",
 					   NULL);
-		if (action == GOO_PLAYER_ACTION_PLAY)
+		if (action == GOO_PLAYER_ACTION_PLAY) {
+			set_current_song_icon (window, GOO_STOCK_PLAY);
 			priv->next_timeout_handle = g_timeout_add (IDLE_TIMEOUT, next_time_idle, window);
+		} else if (action == GOO_PLAYER_ACTION_STOP) {
+			set_current_song_icon (window, GOO_STOCK_STOP);
+		}
 
 		break;
 	case GOO_PLAYER_ACTION_PAUSE:
+		set_current_song_icon (window, GOO_STOCK_PAUSE);
 		set_action_label_and_icon (window,
 					   "TogglePlay", 
 					   _("_Play"), 
@@ -1451,6 +1655,8 @@ row_activated_cb (GtkTreeView       *tree_view,
 	goo_window_stop (window);
 	goo_window_set_current_song (window, song->number);
 	goo_window_play (window);
+
+	song_info_unref (song);
 }
 
 
@@ -1522,6 +1728,68 @@ file_button_press_cb (GtkWidget      *widget,
 }
 
 
+static int
+get_column_from_sort_method (WindowSortMethod sort_method)
+{
+	switch (sort_method) {
+	case WINDOW_SORT_BY_NUMBER: return COLUMN_NUMBER;
+	case WINDOW_SORT_BY_TIME: return COLUMN_TIME;
+	case WINDOW_SORT_BY_TITLE: return COLUMN_TITLE;
+	default: 
+		break;
+	}
+
+	return COLUMN_NUMBER;
+}
+
+
+static int
+get_sort_method_from_column (int column_id)
+{
+	switch (column_id) {
+	case COLUMN_NUMBER: return WINDOW_SORT_BY_NUMBER;
+	case COLUMN_TIME: return WINDOW_SORT_BY_TIME;
+	case COLUMN_TITLE: return WINDOW_SORT_BY_TITLE;
+	default: 
+		break;
+	}
+
+	return WINDOW_SORT_BY_NUMBER;
+}
+
+
+static void
+sort_column_changed_cb (GtkTreeSortable *sortable,
+			gpointer         user_data)
+{
+	GooWindow *window = user_data;
+	GooWindowPrivateData  *priv = window->priv;
+	GtkSortType    order;
+	int            column_id;
+	GooPlayerState state;
+
+	if (! gtk_tree_sortable_get_sort_column_id (sortable, 
+						    &column_id, 
+						    &order))
+		return;
+
+	priv->sort_method = get_sort_method_from_column (column_id);
+	priv->sort_type = order;
+
+	state = goo_player_get_state (priv->player);
+
+	/* Recreate the playlist if not already playing. */
+	if ((state != GOO_PLAYER_STATE_PLAYING) &&  (state != GOO_PLAYER_STATE_PAUSED)) { 
+		gboolean   play_all;
+		gboolean   shuffle;
+		
+		play_all = eel_gconf_get_boolean (PREF_PLAYLIST_PLAYALL, TRUE);
+		shuffle = eel_gconf_get_boolean (PREF_PLAYLIST_SHUFFLE, FALSE);
+		create_playlist (window, play_all, shuffle);
+	}
+}
+
+
 static void
 update_ui_from_expander_state (GooWindow *window)
 {
@@ -1727,6 +1995,7 @@ goo_window_init (GooWindow *window)
 	window->priv->exiting = FALSE;
 	window->priv->check_id = 0;
 	window->priv->url_list = NULL;
+	window->priv->hibernate = FALSE;
 }
 
 
@@ -1775,7 +2044,7 @@ goo_window_construct (GooWindow  *window,
 	/* * File list. */
 
 	priv->list_store = gtk_list_store_new (NUMBER_OF_COLUMNS, 
-					       G_TYPE_POINTER,
+					       GOO_TYPE_SONG_INFO,
 					       G_TYPE_INT,
 					       GDK_TYPE_PIXBUF,
 					       G_TYPE_STRING,
@@ -1800,12 +2069,11 @@ goo_window_construct (GooWindow  *window,
 					 COLUMN_TIME, time_column_sort_func,
 					 NULL, NULL);
 
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (priv->list_store), COLUMN_NUMBER, GTK_SORT_ASCENDING);
-
-	/*
-	gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (window->list_store), default_sort_func,  NULL, NULL);
-	*/
+	priv->sort_method = preferences_get_sort_method ();
+	priv->sort_type = preferences_get_sort_type ();
 
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (priv->list_store), get_column_from_sort_method (priv->sort_method), priv->sort_type);
+	
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->list_view));
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_MULTIPLE);
 
@@ -1822,14 +2090,14 @@ goo_window_construct (GooWindow  *window,
 			  G_CALLBACK (file_button_press_cb), 
 			  window);
 
-	/*
-	egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (window->list_view));
-
-	g_signal_connect (G_OBJECT (window->list_store), 
+	g_signal_connect (G_OBJECT (priv->list_store), 
 			  "sort_column_changed",
 			  G_CALLBACK (sort_column_changed_cb), 
 			  window);
 
+	/*
+	egg_tree_multi_drag_add_drag_support (GTK_TREE_VIEW (window->list_view));
+
         gtk_drag_source_set (window->list_view, 
 			     GDK_BUTTON1_MASK | GDK_BUTTON2_MASK,
 			     target_table, n_targets, 
@@ -2254,6 +2522,9 @@ goo_window_play (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
 
+	if (window->priv->hibernate)
+		return;
+	
 	if (goo_player_get_state (priv->player) == GOO_PLAYER_STATE_PLAYING) 
 		return;
 
@@ -2266,13 +2537,15 @@ goo_window_play (GooWindow *window)
 		create_playlist (window, play_all, shuffle);
 
 		if (priv->current_song != NULL) 
-			goo_player_seek_song (priv->player, priv->current_song->number);
+			play_song (window, priv->current_song->number);
 		else if (window->priv->playlist != NULL)
 			play_next_song_in_playlist (window);
 		else
-			goo_player_seek_song (priv->player, 0);
-	} else
+			play_song (window, 0);
+	} else {
+		set_current_song_icon (window, GOO_STOCK_PLAY);
 		goo_player_play (priv->player);
+	}
 }
 
 
@@ -2298,6 +2571,10 @@ void
 goo_window_stop (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
+
+	if (window->priv->hibernate)
+		return;
+
 	goo_player_stop (priv->player);
 }
 
@@ -2306,6 +2583,10 @@ void
 goo_window_pause (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
+
+	if (window->priv->hibernate)
+		return;
+
 	goo_player_pause (priv->player);
 }
 
@@ -2314,6 +2595,7 @@ void
 goo_window_toggle_play (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
+
 	if (goo_player_get_state (priv->player) != GOO_PLAYER_STATE_PLAYING)
 		goo_window_play (window);
 	else
@@ -2326,14 +2608,22 @@ goo_window_next (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
 
-	if (priv->songs == 0)
+	if (window->priv->songs == 0)
 		return;
 
 	if (priv->current_song != NULL) {
 		if (window->priv->playlist == NULL) {
-			int n = MIN (priv->current_song->number + 1, priv->songs - 1);
+			int current_track = priv->current_song->number;
+			int current_pos;
+			int new_pos, new_track;
+
 			goo_window_stop (window);
-			goo_window_set_current_song (window, n);
+
+			current_pos = get_position_from_track_number (window, current_track);
+			new_pos = MIN (current_pos + 1, priv->songs - 1);
+			new_track = get_track_number_from_position (window, new_pos);
+			goo_window_set_current_song (window, new_track);
+
 			goo_window_play (window);
 		} else
 			play_next_song_in_playlist (window);
@@ -2349,18 +2639,24 @@ void
 goo_window_prev (GooWindow *window)
 {
 	GooWindowPrivateData *priv = window->priv;
-	int n;
+	int new_pos, new_track;
 
 	if (priv->songs == 0)
 		return;
 
-	if (priv->current_song != NULL)
-		n = MAX ((gint)priv->current_song->number - 1, 0);
-	else
-		n = priv->songs - 1;
-
 	goo_window_stop (window);
-	goo_window_set_current_song (window, n);
+
+	if (priv->current_song != NULL) {
+		int current_track, current_pos;
+		current_track = priv->current_song->number;
+		current_pos = get_position_from_track_number (window, current_track);
+		new_pos = MAX (current_pos - 1, 0);
+	} else
+		new_pos = priv->songs - 1;
+
+	new_track = get_track_number_from_position (window, new_pos);
+	goo_window_set_current_song (window, new_track);
+
 	goo_window_play (window);
 }
 
@@ -2368,6 +2664,9 @@ goo_window_prev (GooWindow *window)
 void
 goo_window_eject (GooWindow *window)
 {
+	if (window->priv->hibernate)
+		return;
+
 	if (!goo_player_eject (window->priv->player)) {
 		GError *e = goo_player_get_error (window->priv->player);
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
@@ -2385,6 +2684,9 @@ goo_window_set_location (GooWindow   *window,
 		_gtk_error_dialog_from_gerror_run (GTK_WINDOW (window), 
 						   _("Could not read drive"), &e);
 	}
+
+	if (device_path == NULL)
+		window_update_sensitivity (window);
 }
 
 
@@ -2400,7 +2702,6 @@ add_selected_song (GtkTreeModel *model,
 	gtk_tree_model_get (model, iter,
 			    COLUMN_SONG_INFO, &song,
 			    -1);
-	song_info_ref (song);
 	*list = g_list_prepend (*list, song);
 }
 
@@ -2478,6 +2779,9 @@ goo_window_edit_cddata (GooWindow *window)
 	CORBA_Environment     ev;
 	const char           *discid;
 
+	if (window->priv->hibernate)
+		return;
+
 	discid = goo_player_cd_get_discid (GOO_PLAYER_CD (window->priv->player));
 	
 	if (discid == NULL) 
@@ -2540,6 +2844,9 @@ goo_window_set_cover_image (GooWindow  *window,
 	GdkPixbuf *image;
 	GError    *error = NULL;
 
+	if (window->priv->hibernate)
+		return;
+
 	image = gdk_pixbuf_new_from_file_at_size (filename, 
 						  COVER_SIZE - 2, 
 						  COVER_SIZE - 2, 
@@ -2633,6 +2940,9 @@ goo_window_pick_cover_from_disk (GooWindow *window)
 	GtkFileFilter *filter;
 	char          *path;
 
+	if (window->priv->hibernate)
+		return;
+
 	file_sel = gtk_file_chooser_dialog_new (_("Choose CD Cover Image"),
 						GTK_WINDOW (window),
 						GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -2686,6 +2996,9 @@ goo_window_search_cover_on_internet (GooWindow *window)
 {
 	const char *album, *artist;
 
+	if (window->priv->hibernate)
+		return;
+
 	debug (DEBUG_INFO, "SEARCH ON INTERNET\n");
 
 	album = goo_player_cd_get_album (GOO_PLAYER_CD (window->priv->player));
@@ -2706,6 +3019,9 @@ void
 goo_window_remove_cover (GooWindow   *window)
 {
 	char *cover_filename;
+
+	if (window->priv->hibernate)
+		return;
 		
 	cover_filename = goo_window_get_cover_filename (window);
 	if (cover_filename == NULL)
@@ -2731,7 +3047,7 @@ goo_window_toggle_visibility (GooWindow *window)
 					   _("_Show Window"), 
 					   _("Show the main window"),
 					   NULL,
-					   "/MenuBar/View/",
+					   /*"/MenuBar/View/",  FIXME*/
 					   "/TrayPopupMenu/",
 					   NULL);
 
@@ -2745,7 +3061,7 @@ goo_window_toggle_visibility (GooWindow *window)
 					   _("_Hide Window"), 
 					   _("Hide the main window"),
 					   NULL,
-					   "/MenuBar/View/",
+					   /*"/MenuBar/View/", FIXME*/
 					   "/TrayPopupMenu/",
 					   NULL);
 	}
@@ -2768,7 +3084,30 @@ goo_window_set_volume (GooWindow   *window,
 {
 	GooVolumeToolButton *volume_button;
 
+	if (window->priv->hibernate)
+		return;
+
 	volume_button = GOO_VOLUME_TOOL_BUTTON (window->priv->volume_button);
 	goo_volume_tool_button_set_volume (volume_button, value, TRUE);
 }
 
+
+void
+goo_window_set_hibernate (GooWindow   *window,
+			  gboolean     hibernate)
+{
+	window->priv->hibernate = hibernate;
+
+	if (hibernate) {
+		goo_window_set_location (window, NULL);
+		
+	} else {
+		char *device;
+
+		device = eel_gconf_get_string (PREF_GENERAL_DEVICE, NULL);
+		goo_window_set_location (window, device);
+		g_free (device);
+		
+		goo_window_update (window);
+	}
+}
