@@ -41,6 +41,14 @@ static GtkActionEntry action_entries[] = {
 	  N_("_About"), NULL,
 	  N_("Information about the program"),
 	  G_CALLBACK (activate_action_about) },
+	{ "Contents", GTK_STOCK_HELP,
+	  N_("_Contents"), "F1",
+	  N_("Display the manual"),
+	  G_CALLBACK (activate_action_manual) },
+	{ "Shortcuts", NULL,
+	  N_("_Keyboard shortcuts"), "",
+	  " ",
+	  G_CALLBACK (activate_action_shortcuts) },
 	{ "TogglePlay", GOO_STOCK_PLAY,
 	  N_("_Play"), "space",
 	  N_("Play/Pause"),
@@ -106,7 +114,7 @@ static GtkActionEntry action_entries[] = {
 	  N_("Remove current CD cover"),
 	  G_CALLBACK (activate_action_remove_cover) },
 	{ "ToggleVisibility", NULL,
-	  N_("_Hide Window"), "H",
+	  N_("_Hide Window"), NULL /*"H"*/,
 	  N_("Hide the main window"),
 	  G_CALLBACK (activate_action_toggle_visibility) },
 };
@@ -175,10 +183,11 @@ static const gchar *ui_info =
 "    <menu action='ViewMenu'>"
 "      <menuitem action='ViewToolbar'/>"
 "      <menuitem action='ViewStatusbar'/>"
-"      <separator name='sep01'/>"
-"      <menuitem action='ToggleVisibility'/>"
 "    </menu>"
 "    <menu action='HelpMenu'>"
+"      <menuitem action='Contents'/>"
+"      <menuitem action='Shortcuts'/>"
+"      <separator name='sep01'/>"
 "      <menuitem action='About'/>"
 "    </menu>"
 "  </menubar>"
@@ -186,10 +195,7 @@ static const gchar *ui_info =
 "    <toolitem action='Play'/>"
 "    <toolitem action='Pause'/>"
 "    <toolitem action='Stop'/>"
-"    <separator name='sep01'/>"
-"    <toolitem action='Prev'/>"
 "    <toolitem action='Next'/>"
-"    <separator name='sep02'/>"
 "    <toolitem action='Eject'/>"
 "  </toolbar>"
 "  <popup name='ListPopupMenu'>"
